#include <gtk/gtk.h>

#include "gui.h"

static gboolean delete_event(GtkWidget* w, GdkEvent *e, gpointer d) {
    gtk_main_quit();
    return 0;
}

static gboolean submit_event(GtkWidget* w, GdkEvent *e, gpointer d) {
    commit_rc();
    gtk_main_quit();
    return 0;
}

widget* gui_init(int* argc, char*** argv, char* title) {
    GtkWidget* window;
    gtk_init(argc, argv);
    window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
    gtk_window_set_title(GTK_WINDOW(window), title);
    g_signal_connect(G_OBJECT (window), "delete_event",
                     G_CALLBACK (delete_event), NULL);
    g_signal_connect(G_OBJECT (window), "destroy",
                     G_CALLBACK (delete_event), NULL);
    gtk_container_set_border_width(GTK_CONTAINER(window), 3);
    return window;
}

widget* gui_label_new(char* label) {
    widget* w = gtk_label_new(label);
    gtk_widget_show(w);
    return w;
}

widget* gui_button_new(char* label) {
    widget* w = gtk_button_new_with_label(label);
    g_signal_connect (w, "clicked", G_CALLBACK(submit_event), NULL);
    gtk_widget_show(w);
    return w;
}

widget* gui_text_new(const char* value) {
    widget* w = gtk_entry_new();
    if (value) gtk_entry_set_text(w, value);
    gtk_widget_show(w);
    return w;
}

widget* gui_select_new(char** list, int size, int selected) {
    widget* w = gtk_combo_box_new_text();
    int i;
    for (i = 0; i < size; i++) {
        gtk_combo_box_append_text(w, list[i]);
    }
    gtk_combo_box_set_active(w, selected);
    gtk_widget_show(w);
    return w;
}

widget* gui_checkbox_new(int checked) {
    widget* w = gtk_check_button_new();
    if (checked) gtk_toggle_button_set_active(w, checked);
    gtk_widget_show(w);
    return w;
}

widget* gui_radio_new(widget* r, int checked) {
    widget* w = gtk_radio_button_new_from_widget(r);
    if (checked) gtk_toggle_button_set_active(w, checked);
    gtk_widget_show(w);
    return w;
}

widget* gui_img_new(const char* file) {
    widget* w = gtk_image_new_from_file(file);
    gtk_widget_show(w);
    return w;
}

const char* gui_text_get(widget* w) {
    return gtk_entry_get_text(w);
}

int gui_checkbox_get(widget* w) {
    return gtk_toggle_button_get_active(w);
}

int gui_radio_get(widget* w) {
    GSList* l = gtk_radio_button_group(w);
    int i;
    int len = g_slist_length(l);
    for (i = 0; l; i++, l = g_slist_next(l)) {
        widget* r = g_slist_nth_data(l, 0);
        if (gtk_toggle_button_get_active(r)) return len-i-1;
    }
    return 0;
}

int gui_select_get(widget* w) {
    return gtk_combo_box_get_active(w);
}

widget* gui_frame_new(widget* p, const char* label) {
    widget* w = gtk_frame_new(label);
    gtk_container_add(w, p);
    gtk_widget_show(w);
    return w;
}

widget* gui_vbox_new(int size) {
    widget* w = (widget*)gtk_vbox_new(FALSE, size);
    gtk_widget_show(w);
    return w;
}

widget* gui_hbox_new(int size) {
    widget* w = (widget*)gtk_hbox_new(FALSE, size);
    gtk_widget_show(w);
    return w;
}

widget* gui_table_new(int x, int y) {
    widget* w = (widget*)gtk_table_new(x, y, FALSE);
    gtk_widget_show(w);
    return w;
}

void gui_add(widget* parent, widget* w) {
    gtk_container_add(GTK_CONTAINER(parent), w);
}

void gui_pack(widget* box, widget* w) {
    gtk_box_pack_start(GTK_BOX(box), w, FALSE, FALSE, 0);
}

void gui_attach(widget* table, widget* w, int l, int r, int t, int b) {
    gtk_table_attach_defaults(GTK_TABLE(table), w, l, r, t, b);
}

void gui_start(widget* w) {
    gtk_widget_show(w);
    gtk_main();
}

